@@ -83,6 +83,7 @@ class make:
         ws = cc(self.get_server('socket'))
         ws.send(dumps(self.hs_data))
         if loads(ws.recv())['status'] == 'OK':
+            print('connected')
             while True:
                 try:
                     recv = loads(ws.recv())
@@ -378,7 +379,7 @@ class bot:
             else:
                 continue
 
-    def send_text(self, chat_id, custom_text, message_id = None, mention_id = None):
+    def send_text(self, chat_id, custom_text, message_id = None):
         return self.method(
             'sendMessage',
             {
@@ -390,11 +391,11 @@ class bot:
         )
 
     def reply(self, data, custom_text):
-        message = message(data)
+        msg = message(data)
         return self.send_text(
-            message.chat_id(),
+            msg.chat_id(),
             custom_text,
-            message.message_id()
+            msg.message_id()
         )
 
     def send_image(self, chat_id, file, caption = None, message_id = None, thumbnail = None):
@@ -423,6 +424,31 @@ class bot:
             }
         )
 
+    def send_gif(self, chat_id, file, caption = None, message_id = None):
+        req = self.file_upload(file)
+        return self.method(
+            'sendMessage',
+            {
+                'file_inline': {
+                    'file_id': req[0]['id'],
+                    'mime': file.split('.')[-1],
+                    'dc_id': req[0]['dc_id'],
+                    'access_hash_rec': req[1],
+                    'file_name': file if not 'http' in file else f'from_pyrubi_library.{format}',
+                    'width': 80,
+                    'height': 80,
+                    'time': 1,
+                    'size': str(len(get(file).content if 'http' in file else open(file,'rb').read())),
+                    'type': 'Gif',
+                    'is_round': False
+                },
+                'object_guid': chat_id,
+                'rnd': f'{randint(100000,999999999)}',
+                'reply_to_message_id': message_id,
+                'text': caption
+            }
+        )
+
     def send_voice(self, chat_id, file, caption = None, message_id = None, time = 1):
         req = self.file_upload(file)
         return self.method(
@@ -466,6 +492,19 @@ class bot:
             }
         )
 
+    def send_sticker(self, chat_id, message_id = None):
+        choice_sticker = choice(choice(self.method('getMyStickerSets',{})['sticker_sets'])['top_stickers'])
+        print(choice_sticker)
+        return self.method(
+            'sendMessage',
+            {
+                'sticker': choice(choice_sticker['top_stickers']),
+                'object_guid': chat_id,
+                'rnd': f'{randint(100000,999999999)}',
+                'reply_to_message_id': message_id
+            }
+        )
+
     def edit_message(self, chat_id, new_text, message_id):
         return self.method(
             'editMessage',
@@ -476,16 +515,6 @@ class bot:
             }
         )
 
-    def delete_message(self, chat_id, message_ids = [], type = 'Global'):
-        return self.method(
-            'deleteMessages',
-            {
-                'object_guid': chat_id,
-                'message_ids': message_ids,
-                'type': type
-            }
-        )
-
     def forward_message(self, from_chat_id, message_ids, to_chat_id):
         return self.method(
             'forwardMessages',
@@ -497,6 +526,58 @@ class bot:
             }
         )
 
+    def resend_message(self, chat_id, file_inline, caption = None, message_id = None):
+        return self.method(
+            'sendMessage',
+            {
+                'file_inline': file_inline,
+                'object_guid': chat_id,
+                'rnd': f'{randint(100000,999999999)}',
+                'reply_to_message_id': message_id,
+                'text': caption if caption != None else 'This message from pyrubi library!'
+            }
+        )
+
+    def pin_message(self, chat_id, message_id):
+        return self.method(
+            'setPinMessage',
+            {
+                'object_guid': chat_id,
+                'message_id': message_id,
+                'action': 'Pin'
+            }
+        )
+
+    def unpin_message(self, chat_id, message_id):
+        return self.method(
+            'setPinMessage',
+            {
+                'object_guid': chat_id,
+                'message_id': message_id,
+                'action': 'Unpin'
+            }
+        )
+
+    def search_message(self, chat_id, text):
+        return self.method(
+            'searchChatMessages',
+            {
+                'object_guid': chat_id,
+                'search_text': text,
+                'type':'Text'
+            }
+        )
+
+    def delete_message(self, chat_id, message_ids = [], type = 'Global'):
+        return self.method(
+            'deleteMessages',
+            {
+                'object_guid': chat_id,
+                'message_ids': message_ids,
+                'type': type
+            }
+        )
+
     def get_chat_messsages(self, chat_id, middle_message_id):
         return self.method(
             'getMessagesInterval',
@@ -506,12 +587,12 @@ class bot:
             }
         )['messages']
 
-    def get_messages_info(self, chat_id, messages_id):
+    def get_messages_info(self, chat_id, messages_ids = []):
         return self.method(
             'getMessagesByID',
             {
                 'object_guid': chat_id,
-                'message_ids': messages_id
+                'message_ids': messages_ids
             }
         )['messages'][0]
 
@@ -521,6 +602,9 @@ class bot:
     def get_chats(self, start_id = None):
         return self.method('getChats', {'start_id': start_id})
 
+    def search_chats(self, text):
+        return self.method('searchGlobalObjects',{'search_text': text})['objects']
+
     def get_chat_info(self, chat_id):
         if chat_id.startswith('u'): data = 'User'
         elif chat_id.startswith('g'): data = 'Group'
@@ -535,21 +619,55 @@ class bot:
     def get_last_chat_message_id(self, chat_id):
         return self.get_chat_info(chat_id)['chat']['last_message_id']
 
-    def ban_member_from_group(self, group_id, user_guid):
+    def get_group_admins(self, group_id, only_ids = True):
+        data = self.method('getGroupAdminMembers',{'group_guid': group_id})
+        return [i['member_guid'] for i in data['in_chat_members']] if only_ids else data
+
+    def add_member_to_group(self, group_id, member_ids = []):
+        return self.method(
+            'addGroupMembers',
+            {
+                'group_guid': group_id,
+                'member_guids': member_ids
+            }
+        )
+
+    def add_member_to_channel(self, channel_id, member_ids = []):
+        return self.method(
+            'addChannelMembers',
+            {
+                'channel_guid': channel_id,
+                'member_guids': member_ids
+            }
+        )
+
+    def ban_member_from_group(self, group_id, member_id):
         return self.method(
             'banGroupMember',
             {
                 'group_guid': group_id,
-                'member_guid': user_guid,
+                'member_guid': member_id,
                 'action': 'Set'
             }
         )
 
+    def set_group_access(self, group_id, access_list = []):
+        return self.method(
+            'setGroupDefaultAccess',
+            {
+                'access_list': access_list,
+                'group_guid': group_id
+            }
+        )
+
+    def get_group_link(self, group_id):
+        return self.method('getGroupLink',{'group_guid': group_id})['join_link']
+
+    def get_channel_link(self, channel_id):
+        return self.method('getChannelLink',{'channel_guid': channel_id})['join_link']
+
     def join_group(self, group_link):
         return self.method('joinGroup', {'hash_link': group_link.split('/')[-1]})
-				
-    def leave_group(self, group_guid):
-        return self.method('leaveGroup', {'group_guid': group_guid})
 
     def join_channel(self, channel_id):
         return self.method(
@@ -563,6 +681,9 @@ class bot:
     def join_channel_by_link(self, channel_link):
         return self.method('joinChannelByLink', {'hash_link': channel_link.split('/')[-1]})
 
+    def leave_group(self, group_guid):
+        return self.method('leaveGroup', {'group_guid': group_guid})
+
     def leave_channel(self, channel_id):
         return self.method(
             'joinChannelAction',
@@ -572,6 +693,24 @@ class bot:
             }
         )
 
+    def block_user(self, user_id):
+        return self.method(
+            'setBlockUser',
+            {
+                'user_guid': user_id,
+                'action': 'Block'
+            }
+        )
+
+    def unblock_user(self, user_id):
+        return self.method(
+            'setBlockUser',
+            {
+                'user_guid': user_id,
+                'action': 'Unblock'
+            }
+        )
+
     def edit_profile(self, **kwargs):
         if 'username' in list(kwargs.keys()):
             return self.method(
@@ -593,14 +732,14 @@ class bot:
             )
 
     def request_file(self, file):
-        return (self.method(
+        return self.method(
             'requestSendFile',
             {
                 'file_name': str(file.split('/')[-1]),
                 'mime': file.split('.')[-1],
                 'size': Path(file).stat().st_size if not 'http' in file else len(get(file).content)
             }
-        ))
+        )
 
     def file_upload(self, file):
         req = self.request_file(file)
@@ -701,11 +840,11 @@ class bot_async:
         )
 
     async def reply(self, data, custom_text):
-        message = message(data)
+        msg = message(data)
         return await self.send_text(
-            message.chat_id(),
+            msg.chat_id(),
             custom_text,
-            message.message_id()
+            msg.message_id()
         )
 
     async def send_image(self, chat_id, file, caption = None, message_id = None, thumbnail = None):
@@ -734,7 +873,32 @@ class bot_async:
             }
         )
 
-    async def send_voice(self, chat_id, file, caption = None, message_id = None):
+    async def send_gif(self, chat_id, file, caption = None, message_id = None):
+        req = await self.file_upload(file)
+        return await self.method(
+            'sendMessage',
+            {
+                'file_inline': {
+                    'file_id': req[0]['id'],
+                    'mime': file.split('.')[-1],
+                    'dc_id': req[0]['dc_id'],
+                    'access_hash_rec': req[1],
+                    'file_name': file if not 'http' in file else f'from_pyrubi_library.{format}',
+                    'width': 80,
+                    'height': 80,
+                    'time': 1,
+                    'size': str(len(get(file).content if 'http' in file else open(file,'rb').read())),
+                    'type': 'Gif',
+                    'is_round': False
+                },
+                'object_guid': chat_id,
+                'rnd': f'{randint(100000,999999999)}',
+                'reply_to_message_id': message_id,
+                'text': caption
+            }
+        )
+
+    async def send_voice(self, chat_id, file, caption = None, message_id = None, time = 1):
         req = await self.file_upload(file)
         return await self.method(
             'sendMessage',
@@ -745,7 +909,7 @@ class bot_async:
                     'type': 'Voice',
                     'file_name': f'{randint(100, 1000)}.ogg',
                     'size': str(len(get(file).content if 'http' in file else open(file,'rb').read())),
-                    'time': '1',
+                    'time': time,
                     'mime': 'ogg',
                     'access_hash_rec': req[1],
                 },
@@ -777,6 +941,18 @@ class bot_async:
             }
         )
 
+    async def send_sticker(self, chat_id, message_id = None):
+        choice_sticker = choice(choice(await self.method('getMyStickerSets',{})['sticker_sets'])['top_stickers'])
+        return await self.method(
+            'sendMessage',
+            {
+                'sticker': choice(choice_sticker['top_stickers']),
+                'object_guid': chat_id,
+                'rnd': f'{randint(100000,999999999)}',
+                'reply_to_message_id': message_id
+            }
+        )
+
     async def edit_message(self, chat_id, new_text, message_id):
         return await self.method(
             'editMessage',
@@ -787,16 +963,6 @@ class bot_async:
             }
         )
 
-    async def delete_message(self, chat_id, message_ids = [], type = 'Global'):
-        return await self.method(
-            'deleteMessages',
-            {
-                'object_guid': chat_id,
-                'message_ids': message_ids,
-                'type': type
-            }
-        )
-
     async def forward_message(self, from_chat_id, message_ids, to_chat_id):
         return await self.method(
             'forwardMessages',
@@ -808,17 +974,60 @@ class bot_async:
             }
         )
 
-    async def get_messages_info(self, guid, messages_id):
-        return (await self.method(
-            'getMessagesByID',
+    async def resend_message(self, chat_id, file_inline, caption = None, message_id = None):
+        return await self.method(
+            'sendMessage',
             {
-                'object_guid': guid,
-                'message_ids': messages_id
+                'file_inline': file_inline,
+                'object_guid': chat_id,
+                'rnd': f'{randint(100000,999999999)}',
+                'reply_to_message_id': message_id,
+                'text': caption if caption != None else 'This message from pyrubi library!'
             }
-        ))['messages'][0]
+        )
+
+    async def pin_message(self, chat_id, message_id):
+        return await self.method(
+            'setPinMessage',
+            {
+                'object_guid': chat_id,
+                'message_id': message_id,
+                'action': 'Pin'
+            }
+        )
+
+    async def unpin_message(self, chat_id, message_id):
+        return await self.method(
+            'setPinMessage',
+            {
+                'object_guid': chat_id,
+                'message_id': message_id,
+                'action': 'Unpin'
+            }
+        )
+
+    async def search_message(self, chat_id, text):
+        return await self.method(
+            'searchChatMessages',
+            {
+                'object_guid': chat_id,
+                'search_text': text,
+                'type':'Text'
+            }
+        )
+
+    async def delete_message(self, chat_id, message_ids = [], type = 'Global'):
+        return await self.method(
+            'deleteMessages',
+            {
+                'object_guid': chat_id,
+                'message_ids': message_ids,
+                'type': type
+            }
+        )
 
     async def get_chat_messsages(self, chat_id, middle_message_id):
-        return await (self.method(
+        return (await self.method(
             'getMessagesInterval',
             {
                 'object_guid':chat_id,
@@ -826,12 +1035,12 @@ class bot_async:
             }
         ))['messages']
 
-    async def get_messages_info(self, chat_id, messages_id):
+    async def get_messages_info(self, chat_id, messages_ids = []):
         return (await self.method(
             'getMessagesByID',
             {
                 'object_guid': chat_id,
-                'message_ids': messages_id
+                'message_ids': messages_ids
             }
         ))['messages'][0]
 
@@ -841,6 +1050,9 @@ class bot_async:
     async def get_chats(self, start_id = None):
         return await self.method('getChats', {'start_id': start_id})
 
+    async def search_chats(self, text):
+        return (await self.method('searchGlobalObjects',{'search_text': text}))['objects']
+
     async def get_chat_info(self, chat_id):
         if chat_id.startswith('u'): data = 'User'
         elif chat_id.startswith('g'): data = 'Group'
@@ -855,16 +1067,53 @@ class bot_async:
     async def get_last_chat_message_id(self, chat_id):
         return (await self.get_chat_info(chat_id))['chat']['last_message_id']
 
-    async def ban_member_from_group(self, guid, user_guid):
+    async def get_group_admins(self, group_id, only_ids = True):
+        data = await self.method('getGroupAdminMembers', {'group_guid': group_id})
+        return [i['member_guid'] for i in data['in_chat_members']] if only_ids else data
+
+    async def add_member_to_group(self, group_id, member_ids = []):
+        return await self.method(
+            'addGroupMembers',
+            {
+                'group_guid': group_id,
+                'member_guids': member_ids
+            }
+        )
+
+    async def add_member_to_channel(self, channel_id, member_ids = []):
+        return await self.method(
+            'addChannelMembers',
+            {
+                'channel_guid': channel_id,
+                'member_guids': member_ids
+            }
+        )
+
+    async def ban_member_from_group(self, group_id, member_id):
         return await self.method(
             'banGroupMember',
             {
-                'group_guid': guid,
-                'member_guid': user_guid,
+                'group_guid': group_id,
+                'member_guid': member_id,
                 'action': 'Set'
             }
         )
 
+    async def set_group_access(self, group_id, access_list = []):
+        return await self.method(
+            'setGroupDefaultAccess',
+            {
+                'access_list': access_list,
+                'group_guid': group_id
+            }
+        )
+
+    async def get_group_link(self, group_id):
+        return (await self.method('getGroupLink',{'group_guid': group_id}))['join_link']
+
+    async def get_channel_link(self, channel_id):
+        return (await self.method('getChannelLink',{'channel_guid': channel_id}))['join_link']
+
     async def join_group(self, group_link):
         return await self.method('joinGroup', {'hash_link': group_link.split('/')[-1]})
 				
@@ -892,6 +1141,24 @@ class bot_async:
             }
         )
 
+    async def block_user(self, user_id):
+        return await self.method(
+            'setBlockUser',
+            {
+                'user_guid': user_id,
+                'action': 'Block'
+            }
+        )
+
+    async def unblock_user(self, user_id):
+        return await self.method(
+            'setBlockUser',
+            {
+                'user_guid': user_id,
+                'action': 'Unblock'
+            }
+        )
+
     async def edit_profile(self, **kwargs):
         if 'username' in list(kwargs.keys()):
             return await self.method(
@@ -913,14 +1180,14 @@ class bot_async:
             )
 
     async def request_file(self, file):
-        return (await self.method(
+        return await self.method(
             'requestSendFile',
             {
                 'file_name': str(file.split('/')[-1]),
                 'mime': file.split('.')[-1],
                 'size': Path(file).stat().st_size if not 'http' in file else len(get(file).content)
             }
-        ))
+        )
 
     async def file_upload(self, file):
         req = await self.request_file(file)
